#include <ccan/qs_parse/qs_parse.h>
/* Include the C files directly. */
#include <ccan/qs_parse/qs_parse.c>
#include <ccan/tap/tap.h>
#include <stdio.h>

int main(void)
{
    int i,j;
    char * kvpairs[256];
    char value[256];
    unsigned char r, g, b, a;
    double dr, dg, db, da;

    char foo[]        = "foo";
    char foobar[]     = "foobar";
    char foo_enc[]    = "fo%6f";
    char foobar_enc[] = "fo%6fbar";
    char foo_mal[]    = "fo%6";
    char foo_mal2[]    = "foo%6";
    char foo_end[]     = "foo&";

    char getstring[] = "FUNNYJUNK://@?t%65xt=bleh+bleh%20!&font=Vera.ttf&size=40.3&fg=Fa98BF44%f&debug1&bg=0C0&color=FF4455F&hash=24a%62cdef%7a&rot=123.1&tr+i=cky&debug2#don'tparseme,bro";

    plan_tests(62);

    /* Since a great deal of this library rides on the functionality of
     * qs_strncmp(), it will be thoroughly tested.
     */

    ok1(qs_strncmp(foo, foo, 3) == 0);
    ok1(qs_strncmp(foo, foo, 4) == 0);
    ok1(qs_strncmp(foo, foo, 99) == 0);

    ok1(qs_strncmp(foo, foo_enc, 3) == 0);
    ok1(qs_strncmp(foo, foo_enc, 4) == 0);
    ok1(qs_strncmp(foo, foo_enc, 99) == 0);

    ok1(qs_strncmp(foo_enc, foo_enc, 3) == 0);
    ok1(qs_strncmp(foo_enc, foo_enc, 4) == 0);
    ok1(qs_strncmp(foo_enc, foo_enc, 99) == 0);

    ok1(qs_strncmp(foo, foobar, 3) == 0);
    ok1(qs_strncmp(foo, foobar, 4) < 0);
    ok1(qs_strncmp(foo, foobar, 99) < 0);

    ok1(qs_strncmp(foo, foobar_enc, 3) == 0);
    ok1(qs_strncmp(foo, foobar_enc, 4) < 0);
    ok1(qs_strncmp(foo, foobar_enc, 99) < 0);

    ok1(qs_strncmp(foo_enc, foobar_enc, 3) == 0);
    ok1(qs_strncmp(foo_enc, foobar_enc, 4) < 0);
    ok1(qs_strncmp(foo_enc, foobar_enc, 99) < 0);

    ok1(qs_strncmp(foo, foo_mal, 2) == 0);
    ok1(qs_strncmp(foo, foo_mal, 3) > 0);
    ok1(qs_strncmp(foo, foo_mal, 99) > 0);

    ok1(qs_strncmp(foo, foo_mal2, 2) == 0);
    ok1(qs_strncmp(foo, foo_mal2, 3) == 0);
    ok1(qs_strncmp(foo, foo_mal2, 4) == 0);
    ok1(qs_strncmp(foo, foo_mal2, 99) == 0);

    ok1(qs_strncmp(foo, foo_end, 2) == 0);
    ok1(qs_strncmp(foo, foo_end, 3) == 0);
    ok1(qs_strncmp(foo, foo_end, 4) == 0);
    ok1(qs_strncmp(foo, foo_end, 99) == 0);


    ok1((strcmp("bleh bleh !", qs_scanvalue("text", getstring, value, 256)) == 0));
    ok1((strcmp("Vera.ttf", qs_scanvalue("font", getstring, value, 256)) == 0));
    ok1((strcmp("40.3", qs_scanvalue("size", getstring, value, 256)) == 0));
    ok1((strcmp("Fa98BF44", qs_scanvalue("fg", getstring, value, 256)) == 0));
    ok1((strcmp("", qs_scanvalue("debug1", getstring, value, 256)) == 0));
    ok1((strcmp("0C0", qs_scanvalue("bg", getstring, value, 256)) == 0));
    ok1((strcmp("FF4455F", qs_scanvalue("color", getstring, value, 256)) == 0));
    ok1((strcmp("24abcdefz", qs_scanvalue("hash", getstring, value, 256)) == 0));
    ok1((strcmp("123.1", qs_scanvalue("rot", getstring, value, 256)) == 0));
    ok1((strcmp("cky", qs_scanvalue("tr i", getstring, value, 256)) == 0));
    ok1((strcmp("", qs_scanvalue("debug2", getstring, value, 256)) == 0));

    ok1((i = qs_parse(getstring, kvpairs, 256)) == 11);

    ok1((strcmp("bleh bleh !", qs_k2v("text", kvpairs, i)) == 0));
    ok1((strcmp("Vera.ttf", qs_k2v("font", kvpairs, i)) == 0));
    ok1((strcmp("40.3", qs_k2v("size", kvpairs, i)) == 0));
    ok1((strcmp("Fa98BF44", qs_k2v("fg", kvpairs, i)) == 0));
    ok1((strcmp("", qs_k2v("debug1", kvpairs, i)) == 0));
    ok1((strcmp("0C0", qs_k2v("bg", kvpairs, i)) == 0));
    ok1((strcmp("FF4455F", qs_k2v("color", kvpairs, i)) == 0));
    ok1((strcmp("24abcdefz", qs_k2v("hash", kvpairs, i)) == 0));
    ok1((strcmp("123.1", qs_k2v("rot", kvpairs, i)) == 0));
    ok1((strcmp("cky", qs_k2v("tr i", kvpairs, i)) == 0));
    ok1((strcmp("", qs_k2v("debug2", kvpairs, i)) == 0));

    ok1((j = hex2ccolor(qs_k2v("fg", kvpairs, i), &r, &g, &b, &a)) == 8);
    ok1(r==250 && g==152 && b==191 && a==68);
    ok1((j = hex2dcolor(qs_k2v("fg", kvpairs, i), &dr, &dg, &db, &da)) == 8);
    ok1((dr-0.98<0.01) && (dg-0.60<0.01) && (db-0.75<0.01) && (da-0.27<0.01));

    ok1((j = hex2ccolor(qs_k2v("bg", kvpairs, i), &r, &g, &b, &a)) == 3);
    ok1(r==0 && g==204 && b==0);
    ok1((j = hex2dcolor(qs_k2v("bg", kvpairs, i), &dr, &dg, &db, &da)) == 3);
    ok1((dr-0.00<0.01) && (dg-0.80<0.01) && (db-0.00<0.01));

    ok1((j = hex2ccolor(qs_k2v("color", kvpairs, i), &r, &g, &b, &a)) == 0);
    ok1((j = hex2dcolor(qs_k2v("color", kvpairs, i), &dr, &dg, &db, &da)) == 0);
    

/*
    printf("    Testing hex2ccolor() and hex2dcolor() with color (\"%s\"):\n", qs_k2v("color", kvpairs, i));
    j = hex2ccolor(qs_k2v("color", kvpairs, i), &r, &g, &b, &a);
    printf("        hex2ccolor() should have decoded 0 chars : %d chars decoded\n", j);
    j = hex2ccolor(qs_k2v("color", kvpairs, i), &r, &g, &b, &a);
    printf("        hex2dcolor() should have decoded 0 chars : %d chars decoded\n", j);
*/







	/* Same, with an explicit description of the test. */
//	ok(some_test(), "%s with no args should return 1", "some_test")
	/* How to print out messages for debugging. */
//	diag("Address of some_test is %p", &some_test)




	return exit_status();
}
